@@ -6,8 +6,8 @@ Created on 18 Sep 2016
 
 from collections import OrderedDict
 
-from scs_core.data.datum import Format
 from scs_core.data.json import JSONable
+from scs_core.data.str import Str
 
 
 # --------------------------------------------------------------------------------------------------------------------
@@ -57,4 +57,4 @@ class AFEDatum(JSONable):
     # ----------------------------------------------------------------------------------------------------------------
 
     def __str__(self, *args, **kwargs):
-        return "AFEDatum:{pt1000:%s, sns:%s}" % (self.pt1000, Format.collection(self.sns))
+        return "AFEDatum:{pt1000:%s, sns:%s}" % (self.pt1000, Str.collection(self.sns))
