@@ -13,6 +13,7 @@ example JSON:
 from collections import OrderedDict
 
 from scs_core.data.json import PersistentJSONable
+from scs_core.data.str import Str
 
 from scs_core.gas.sensor_baseline import SensorBaseline
 
@@ -33,8 +34,8 @@ class AFEBaseline(PersistentJSONable):
     __FILENAME =    "afe_baseline.json"
 
     @classmethod
-    def persistence_location(cls, host):
-        return host.conf_dir(), cls.__FILENAME
+    def persistence_location(cls):
+        return cls.conf_dir(), cls.__FILENAME
 
 
     # ----------------------------------------------------------------------------------------------------------------
@@ -92,6 +93,4 @@ class AFEBaseline(PersistentJSONable):
     # ----------------------------------------------------------------------------------------------------------------
 
     def __str__(self, *args, **kwargs):
-        sensor_baselines = '[' + ', '.join(str(baseline) for baseline in self.__sensor_baselines) + ']'
-
-        return "AFEBaseline:{sensor_baselines:%s}" % sensor_baselines
+        return "AFEBaseline:{sensor_baselines:%s}" % Str.collection(self.__sensor_baselines)
